@@ -274,8 +274,15 @@ file_clean(void)
                     isp->u.u_f.name, isp->u.u_f.lineno);
 
   fclose (isp->u.u_f.file);
-  xfree (current_file);
-  current_file = xstrdup (isp->u.u_f.name);
+  /*  A common mistake is to forget closing some complex tags, which
+      result in an error `file: EOF when reading body of...'
+      If file name has been changed by <__file__>, this name
+      should appear in error message instead of real file name.  */
+  if (isp->prev != NULL)
+    {
+      xfree (current_file);
+      current_file = xstrdup (isp->u.u_f.name);
+    }
   xfree (isp->u.u_f.name);
   current_line = isp->u.u_f.lineno;
   output_current_line = isp->u.u_f.out_lineno;
@@ -843,9 +850,6 @@ input_init (void)
      varbreak symbol.  */
   init_break ();
 
-  /* This routine tells whether we want extended regexp or not. */
-  set_regexp_extended (TRUE);
-  
   syntax_init ();
 }
 
@@ -1075,7 +1079,10 @@ next_token (token_data *td, read_type expansion, boolean in_string)
         skip_line ();
       else if (expansion == READ_BODY)
         {
-          obstack_1grow (&token_stack, ch);
+          if (ch == '"')
+            obstack_1grow (&token_stack, CHAR_QUOTE);
+          else
+            obstack_1grow (&token_stack, ch);
           while ((ch = next_char ()) != CHAR_EOF
                   && ! IS_ESCAPE(ch) && ! IS_CLOSE (ch))
             {
@@ -1085,7 +1092,10 @@ next_token (token_data *td, read_type expansion, boolean in_string)
                   ch = CHAR_EOF;
                   break;
                 }
-              obstack_1grow (&token_stack, ch);
+              if (ch == '"')
+                obstack_1grow (&token_stack, CHAR_QUOTE);
+              else
+                obstack_1grow (&token_stack, ch);
             }
           unget_input(ch);
 
