@@ -358,6 +358,8 @@ struct token_data
                          (1 << 6)  /* Do not remove trailing slash in simple tag attributes */
 #define EXP_LEAVE_LEADING_STAR \
                          (1 << 7)  /* Do not remove trailing slash in simple tag attributes */
+#define EXP_NOSPACE_BSLASH \
+                         (1 << 8)  /* Do not add space before trailing slash in simple tag attributes */
 #define EXP_NOWARN_NEST  (1 << 10) /* Suppress warning about bad nested tags */
 #define EXP_NOWARN_SLASH (1 << 11) /* Suppress warning about missing trailing slash */
 
@@ -393,7 +395,8 @@ void read_file_verbatim __P ((struct obstack *));
 extern char *current_file;
 extern int current_line;
 extern int *array_current_line;
-#define CURRENT_FILE_LINE current_file, array_current_line[expansion_level]
+#define CURRENT_FILE_LINE current_file, (expansion_level == 0 ? current_line: \
+                                        array_current_line[expansion_level])
 
 /* Begin and end quote */
 extern STRING lquote, rquote;
@@ -472,7 +475,6 @@ extern int output_current_line;
 void output_init __P ((void));
 void output_deallocate __P ((void));
 void shipout_text __P ((struct obstack *, char *));
-void shipout_string __P ((struct obstack *obs, const char *s, int len));
 void make_diversion __P ((int));
 void insert_diversion __P ((int));
 void insert_file __P ((FILE *));
@@ -563,7 +565,6 @@ void builtin_init __P ((void));
 void builtin_deallocate __P ((void));
 void clear_tag_attr __P ((void));
 void define_builtin __P ((const char *, const builtin *, boolean));
-void set_regexp_extended __P ((boolean));
 void init_break __P ((void));
 void define_user_macro __P ((const char *, char *, symbol_lookup,
                              boolean, boolean, boolean));
@@ -573,6 +574,18 @@ void expand_user_macro __P ((struct obstack *, symbol *, int, token_data **,
 
 const builtin *find_builtin_by_addr __P ((builtin_func *));
 const builtin *find_builtin_by_name __P ((const char *));
+void install_builtin_table __P ((builtin *));
+
+
+/* File: devel.c  --- global functions for writing builtins and modules.  */
+
+boolean bad_argc __P ((token_data *, int, int, int));
+boolean numeric_arg __P ((token_data *, const char *, boolean, int *));
+void shipout_int __P ((struct obstack *, int));
+void shipout_long __P ((struct obstack *, long));
+void shipout_string __P ((struct obstack *, const char *, int));
+void dump_args __P ((struct obstack *, int, token_data **, const char *));
+const char * predefined_attribute __P ((const char *, int *, token_data **, boolean));
 
 
 /* File: path.c  --- path search for include files.  */
@@ -631,17 +644,15 @@ void reload_frozen_state __P ((const char *));
 
 #ifdef WITH_MODULES
 
-typedef void module_init_t (struct obstack *obs);
-typedef void module_finish_t (void);
+typedef void module_init_t __P ((struct obstack *));
+typedef void module_finish_t __P ((void));
 
-typedef voidstar module_func (const char *);
+typedef voidstar module_func __P ((const char *));
 
-void module_init (void);
-void module_unload_all (void);
+void module_load __P ((const char *, struct obstack *));
+void module_unload_all __P ((void));
 
-builtin *module_load (const char *modname, struct obstack *obs);
-
-#endif /* WITH_MODULES */
+#endif
 
 
 /* Debugging the memory allocator.  */
@@ -660,4 +671,51 @@ builtin *module_load (const char *modname, struct obstack *obs);
 # define DEBUG_INCL
 #endif
 
+
+/*  Stuff for compiling builtins and loadable modules.  */
+
+#ifdef MP4H_MODULE
+
+#define MP4H_BUILTIN_ARGS struct obstack *obs, int argc, token_data **argv, \
+                            read_type expansion
+#define MP4H_BUILTIN_PROTO struct obstack *, int, token_data **, read_type
+#define MP4H_BUILTIN_RECUR obs, argc, argv, expansion
+
+#define DECLARE(name) \
+  static void name __P ((MP4H_BUILTIN_PROTO))
+
+#define ARG(i)  (i<argc ? TOKEN_DATA_TEXT (argv[i]) : "")
+#define ARGBODY (TOKEN_DATA_TEXT (argv[argc]))
+
+enum mathop_type
+{
+  MATHOP_ADD,                   /* addition */
+  MATHOP_SUB,                   /* substraction */
+  MATHOP_MUL,                   /* multiplication */
+  MATHOP_DIV,                   /* division */
+  MATHOP_MIN,                   /* minimum */
+  MATHOP_MAX,                   /* maximum */
+  MATHOP_MOD                    /* modulus */
+};
+
+enum mathrel_type
+{
+  MATHREL_GT,
+  MATHREL_LT,
+  MATHREL_EQ,
+  MATHREL_NEQ 
+};
+
+typedef enum mathop_type mathop_type;
+typedef enum mathrel_type mathrel_type;
+
+typedef struct var_stack var_stack;
+struct var_stack
+{
+    var_stack *prev;
+    char *text;
+};
+
+#endif
+
 #endif /* MP4H_H */
