@@ -282,14 +282,14 @@ void
 remove_special_chars (char *s, boolean restore_quotes)
 {
   int offset = 0;
-  register char *cp;
+  register unsigned char *cp = (unsigned char *) s;
 
   /*  For efficiency reason, this loop is divided into 2 parts.
       Until a special character is removed, there is nothing to do.
       After that, string is shifyed.  */
   if (restore_quotes)
     {
-      for (cp=s; *cp != '\0' && !(IS_GROUP (*cp)); cp++)
+      for ( ; *cp != '\0' && !(IS_GROUP (*cp)); cp++)
         {
           if (*cp == CHAR_QUOTE)
             *cp = '"';
@@ -297,7 +297,7 @@ remove_special_chars (char *s, boolean restore_quotes)
     }
   else
     {
-      for (cp=s; *cp != '\0' && !(IS_GROUP (*cp)); cp++)
+      for ( ; *cp != '\0' && !(IS_GROUP (*cp)); cp++)
         ;
     }
 
@@ -325,10 +325,9 @@ static void
 remove_stars_and_slash (char *s)
 {
   int offset;
-  register char *cp;
+  register unsigned char *cp = (unsigned char *) s;
 
   offset = 0;
-  cp = s;
   if (*s == '*' && after_left_angle &&
           !(exp_flags & EXP_LEAVE_LEADING_STAR))
     {
@@ -343,7 +342,7 @@ remove_stars_and_slash (char *s)
           if (exp_flags & EXP_REMOVE_TRAILING_SLASH)
             {
               offset++;
-              if (cp > s && IS_SPACE(*(cp-1)))
+              if (cp > (unsigned char *) s && IS_SPACE(*(cp-1)))
                 offset++;
             }
           else
